import click
import json
import os
from pathlib import Path
from cli.utils import exception_handler, wowza_auth, credentials_file_exist

PROFILE_HELP = """
`wowtool` profile, can be set with WOWTOOL_PROFILE env var.
profile settings in ~/.wow/config.json
"""


@click.group(invoke_without_command=True)
def configure():
    """
    Setup credentials file.
    """

    if credentials_file_exist():
        if click.confirm('Credentials file already exist, override?'):
            wowza_auth()
            click.echo('Credentials file successfully created!')
    else: 
        wowza_auth()
        click.echo('Credentials file successfully created!')



import json
import click
import sys
import yaml
from click_option_group import optgroup, RequiredMutuallyExclusiveOptionGroup
from cli.utils import exception_handler, wowza_auth
from cli.database import Database
from wowpy.resources import get_resource_info
from wowpy.resources import validate_resource, create_resource, delete_resource
from pygments import highlight
from pygments.lexers import JsonLexer
from pygments.formatters import TerminalTrueColorFormatter

PROFILE_HELP = """
`wowtool` profile, can be set with WOWTOOL_PROFILE env var.
profile settings in ~/.wow/config.json
"""


@click.group()
def resource():
    """
    Resource commands.
    """
    pass


@resource.command(name='create')
@click.option('--spec-file', required=True, type=click.File(lazy=False),
              help='File that contains your streaming resource specification')
@exception_handler
def create(spec_file):
    """Create streaming resource"""
    specification = yaml.load(spec_file, Loader=yaml.SafeLoader)
    valid = validate_resource(specification)
    if not valid:
        click.echo('The specification template is invalid')
    else:
        click.echo('The specification template is valid')
        
    wowza_auth()
    response = create_resource(specification)
    formatted_json = json.dumps(response, indent=4)
    colorful_json = highlight(formatted_json, JsonLexer(), TerminalTrueColorFormatter(style='solarized-dark'))
    click.echo(colorful_json)

@resource.command(name='delete')
@optgroup.group('Resource identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for resource')
@optgroup.option('--id', help='Resource stream id')
@optgroup.option('--name', help='Resource stream name')
@exception_handler
def delete(id, name):
    """Delete streaming resource"""  
    wowza_auth()
    if name:
        database = Database()
        if not database.live_streams_table_exist():
            click.echo('In order to use --name parameter please run: wowtool mapper live_streams, first')
            sys.exit()

        id = database.get_live_stream_id_from_name(name)
        if not id:
            click.echo('Live stream not found in current mapping table')
            sys.exit()
    
    delete_resource(id)
    click.echo('Streaming resource deleted')


@resource.command(name='validator')
@click.option('--spec-file', required=True, type=click.File(lazy=False),
              help='File that contains your streaming resource specification')
@exception_handler
def validator(spec_file):
    """Validate streaming resource specification"""
    specification = yaml.load(spec_file, Loader=yaml.SafeLoader)
    valid = validate_resource(specification)
    if not valid:
        click.echo('The specification template is invalid')
    else:
        click.echo('The specification template is valid')


@resource.command(name='query')
@optgroup.group('Resource identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for resource')
@optgroup.option('--id', help='Resource stream id')
@optgroup.option('--name', help='Resource stream name')
@exception_handler
def query(id, name):
    """Get data for a resource"""
    wowza_auth()
    if name:
        database = Database()
        if not database.live_streams_table_exist():
            click.echo('In order to use --name parameter please run: wowtool mapper live_streams, first')
            sys.exit()

        id = database.get_live_stream_id_from_name(name)
        if not id:
            click.echo('Live stream not found in current mapping table')
            sys.exit()
    response = get_resource_info(id)
    formatted_json = json.dumps(response, indent=4)
    colorful_json = highlight(formatted_json, JsonLexer(), TerminalTrueColorFormatter(style='solarized-dark'))
    click.echo(colorful_json)


import json
import click
import sys
from click_option_group import optgroup, RequiredMutuallyExclusiveOptionGroup
from wowpy.transcoders import Transcoder
from cli.utils import exception_handler, wowza_auth
from cli.database import Database
from pygments import highlight
from pygments.lexers import JsonLexer
from pygments.formatters import TerminalTrueColorFormatter

PROFILE_HELP = """
`wowtool` profile, can be set with WOWTOOL_PROFILE env var.
profile settings in ~/.wow/config.json
"""


@click.group()
def trans():
    """
    Transcoder commands.
    """
    pass


@trans.command(name='query')
@optgroup.group('Transcoder identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for wowza transcoder')
@optgroup.option('--id', help='Transcoder id')
@optgroup.option('--name', help='Transcoder name')
@exception_handler
def query(id, name):
    """Get data for a Transcoder"""
    wowza_auth()
    if name:
        database = Database()
        if not database.live_streams_table_exist():
            click.echo('In order to use --name parameter please run: wowtool mapper live_streams (this maps also transcoders), first')
            sys.exit()

        id = database.get_live_stream_id_from_name(name)
        if not id:
            click.echo('Transcoder not found in current mapping table')
            sys.exit()
    response = Transcoder.get_transcoder(id)
    formatted_json = json.dumps(response, indent=4)
    colorful_json = highlight(formatted_json, JsonLexer(), TerminalTrueColorFormatter(style='solarized-dark'))
    click.echo(colorful_json)

@trans.command(name='start')
@optgroup.group('Transcoder identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for wowza transcoder')
@optgroup.option('--id', help='Transcoder id')
@optgroup.option('--name', help='Transcoder name')
@exception_handler
def start(id, name):
    """Start a Transcoder"""
    wowza_auth()
    Transcoder.start_transcoder(id)
    click.echo('Transcoder started')

@trans.command(name='stop')
@optgroup.group('Transcoder identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for wowza transcoder')
@optgroup.option('--id', help='Transcoder id')
@optgroup.option('--name', help='Transcoder name')
@exception_handler
def stop(id, name):
    """Stop a Transcoder"""
    wowza_auth()
    Transcoder.stop_transcoder(id)
    click.echo('Transcoder stopped')

@trans.command(name='reset')
@optgroup.group('Transcoder identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for wowza transcoder')
@optgroup.option('--id', help='Transcoder id')
@optgroup.option('--name', help='Transcoder name')
@exception_handler
def reset(id, name):
    """Reset a Transcoder"""
    wowza_auth()
    Transcoder.reset_transcoder(id)
    click.echo('Transcoder reset')


import json
from pkg_resources import resource_string

# Load config
cli_config = resource_string(__name__, 'config.json')
cli_settings = json.loads(cli_config)['properties']

DATABASE_FILENAME = cli_settings['DATABASE_FILENAME']
CREDENTIALS_FILENAME = cli_settings['CREDENTIALS_FILENAME']
CONFIG_PATH = cli_settings['CONFIG_PATH']
LOG_LEVEL = cli_settings['LOG_LEVEL']


import pytest
from cli.database import Database

@pytest.fixture
def database():
    database = Database(database_file_path='tests/fixtures/test_database.json')
    return database

def test_live_streams_table_exist(mocker, database):
    exist = database.live_streams_table_exist()
    assert exist in [False, True]

def test_get_live_stream_id_from_name(mocker, database):
    id = database.get_live_stream_id_from_name('Baton-6945748')
    assert id=='ww7jxjtw'


import functools
import click
import datetime
import json
import os
from cli.constants import CONFIG_PATH, DATABASE_FILENAME, CREDENTIALS_FILENAME
from pathlib import Path

def exception_handler(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            message = 'There was an error in the CLI: {}'.format(str(e))
            click.echo(message, err=True)
            exit(1)

    return wrapper

# core auth

def wowza_auth():
    if credentials_file_exist():
        __set_credentials_envars()
    else:
        __create_creds_file()
        __set_credentials_envars()

def __set_credentials_envars():
    credentials_file_path = get_base_file_path(filename=CREDENTIALS_FILENAME)
    with open(credentials_file_path) as json_file:
        credentials_data = json.load(json_file)
        os.environ["WSC_ACCESS_KEY"] = credentials_data['WSC_ACCESS_KEY']
        os.environ["WSC_API_KEY"] = credentials_data['WSC_API_KEY']

def __create_creds_file():
    config_path=__get_config_path()
    wsc_access_key = click.prompt('Please enter wowza access key')
    wsc_api_key = click.prompt('Please enter wowza api key')
    keys = {
            'WSC_ACCESS_KEY': wsc_access_key,
            'WSC_API_KEY': wsc_api_key
            }
    Path(config_path).mkdir(parents=True, exist_ok=True)
    credentials_file_path = get_credentials_file_path()
    with open(credentials_file_path, 'w') as json_file:
        json_file.write(json.dumps(keys, indent=4))

def credentials_file_exist():
    exist = config_file_exist(filename=CREDENTIALS_FILENAME)
    return exist

def get_credentials_file_path():
    file_path = get_base_file_path(CREDENTIALS_FILENAME)
    return file_path

# core paths

def __get_config_path():
    home_path = str(Path.home())
    config_path = home_path + CONFIG_PATH
    return config_path

# def __get_config_filename():
#     config_filename = DATABASE_FILENAME
#     return config_filename

def config_file_exist(filename, config_path=__get_config_path()):
    base_file_path = get_base_file_path(filename, config_path=config_path)
    if os.path.exists(base_file_path):
        return True
    else:
        return False

def get_base_file_path(filename, config_path=__get_config_path()):
    full_file_path = config_path + filename
    return full_file_path



import logging
from logging import Logger
import click
from cli.__version__ import VERSION
from cli.constants import LOG_LEVEL
from cli.commands.livestream import live
from cli.commands.transcoder import trans
from cli.commands.configure import configure
from cli.commands.resource import resource 
from cli.commands.recording import recording
from cli.commands.mapping import mapper

class NaturalOrderGroup(click.Group):
    # Link: https://github.com/pallets/click/issues/513
    def list_commands(self, ctx):
        return self.commands.keys()

@click.group(cls=NaturalOrderGroup)
@click.version_option(version=VERSION)
def cli():
    pass

cli.add_command(mapper)
cli.add_command(resource)
cli.add_command(live)
cli.add_command(trans)
cli.add_command(recording)
cli.add_command(configure)

# Logger setup
logger = Logger('')
logger.setLevel(LOG_LEVEL)
ch = logging.StreamHandler()
ch.setLevel(LOG_LEVEL)
logger.addHandler(ch)


import json
import click
from tinydb import TinyDB
from click_option_group import optgroup, RequiredMutuallyExclusiveOptionGroup
from cli.utils import exception_handler, wowza_auth 
from cli.database import Database
from wowpy.mappings import Mapper

PROFILE_HELP = """
`wowtool` profile, can be set with WOWTOOL_PROFILE env var.
profile settings in ~/.wow/config.json
"""

# TODO: Create a map all, taking into account some resources like targets and recordings will be null

@click.group()
def mapper():
    """
    Mapping of ids to resource names
    """
    pass


@mapper.command(name='live_streams')
@exception_handler
def live_streams():
    """Map live stream ids with resource names"""
    wowza_auth()
    document_batch = Mapper.map_live_streams()
    database = Database()
    database.insert_live_streams_document(document_batch)
    click.echo('Mapping of live stream ids with resource names done')

@mapper.command(name='recordings')
@exception_handler
def recordings():
    """Map recording ids with resource names"""
    wowza_auth()
    document_batch = Mapper.map_recordings()
    database = Database()
    database.insert_recordings_document(document_batch)
    click.echo('Mapping of recording ids with resource names done')



import subprocess

# TODO: These are integration tests

def test_cli_version():
    # Link: https://janakiev.com/blog/python-shell-commands/
    process = subprocess.run(['wowtool', '--version'], 
                            stdin =subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, 
                            universal_newlines=True)
    cli_response = process.stderr
    assert 'There was an error in the CLI' not in cli_response

def test_cli_recording_query():
    process = subprocess.run(['wowtool', 'recording', 'query', '--name', 'Baton-7124602'], 
                            stdin =subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, 
                            universal_newlines=True)
    cli_response = process.stderr
    assert 'There was an error in the CLI' not in cli_response

def test_cli_live_query():
    process = subprocess.run(['wowtool', 'live', 'query', '--name', 'Baton-7124602'], 
                            stdin =subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, 
                            universal_newlines=True)
    cli_response = process.stderr
    assert 'There was an error in the CLI' not in cli_response

def test_cli_trans_query():
    process = subprocess.run(['wowtool', 'trans', 'query', '--name', 'Baton-7083486'], 
                            stdin =subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, 
                            universal_newlines=True)
    cli_response = process.stderr
    assert 'There was an error in the CLI' not in cli_response

def test_cli_resource_validator():
    process = subprocess.run(['wowtool', 'resource', 'validator', '--spec-file', 'tests/fixtures/single_output_spec.yaml'], 
                            stdin =subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, 
                            universal_newlines=True)
    cli_response = process.stderr
    assert 'There was an error in the CLI' not in cli_response